@@ -60,12 +60,15 @@
 #include        "string.h"              /* strings                         */
 
 #include        "code.h"                /* coder                           */
-#include        "vars.h"                /* variables                       */
 
 #include        "stats.h"               /* statements                      */
 
 #include        "saveload.h"            /* saving and loading              */
+#include        "tls.h"                 /* thread-local storage            */
 
+#include        "vars.h"                /* variables                       */
+
+#include <assert.h>
 
 /****************************************************************************
 **
@@ -274,19 +277,38 @@ libGAP_Obj libGAP_DoWrap6args (
 
 *F  DoFail0args( <self> )  . . . . . .  fail a function call with 0 arguments
 **
-**  'DoWrap<i>args' accepts the <i> arguments <arg1>, <arg2>,  and so on, and
+**  'DoFail<i>args' accepts the <i> arguments <arg1>, <arg2>,  and so on, and
 **  signals an error,  because  the  function for  which  they  are installed
 **  expects another number of arguments.  'DoFail<i>args' are the handlers in
 **  the other slots of a function.
 */
+
+/* Pull this out to avoid repetition, since it gets a little more complex in 
+   the presence of partially variadic functions */
+
+libGAP_Obj libGAP_NargError( libGAP_Obj func, libGAP_Int actual) {
+  libGAP_Int narg = libGAP_NARG_FUNC(func);
+
+  if (narg >= 0) {
+    assert(narg != actual);
+    return libGAP_ErrorReturnObj(
+			  "Function: number of arguments must be %d (not %d)",
+			  narg, actual,
+			  "you can replace the argument list <args> via 'return <args>;'" );
+  } else {
+    assert(-narg-1 > actual);
+    return libGAP_ErrorReturnObj(
+        "Function: number of arguments must be at least %d (not %d)",
+        -narg-1, actual,
+        "you can replace the argument list <args> via 'return <args>;'" );
+  }
+}
+
 libGAP_Obj libGAP_DoFail0args (
     libGAP_Obj                 self )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), 0L,
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, 0);
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -300,10 +322,7 @@ libGAP_Obj libGAP_DoFail1args (
     libGAP_Obj                 arg1 )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), 1L,
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, 1);
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -318,10 +337,7 @@ libGAP_Obj libGAP_DoFail2args (
     libGAP_Obj                 arg2 )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), 2L,
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, 2);
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -337,10 +353,7 @@ libGAP_Obj libGAP_DoFail3args (
     libGAP_Obj                 arg3 )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), 3L,
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, 3);
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -357,10 +370,7 @@ libGAP_Obj libGAP_DoFail4args (
     libGAP_Obj                 arg4 )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), 4L,
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, 4);
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -378,10 +388,7 @@ libGAP_Obj libGAP_DoFail5args (
     libGAP_Obj                 arg5 )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), 5L,
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, 5);
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -400,10 +407,7 @@ libGAP_Obj libGAP_DoFail6args (
     libGAP_Obj                 arg6 )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), 6L,
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, 6);
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -417,10 +421,7 @@ libGAP_Obj libGAP_DoFailXargs (
     libGAP_Obj                 args )
 {
     libGAP_Obj                 argx;           /* arguments list (to continue)    */
-    argx = libGAP_ErrorReturnObj(
-        "Function: number of arguments must be %d (not %d)",
-        (libGAP_Int)libGAP_NARG_FUNC( self ), libGAP_LEN_LIST( args ),
-        "you can replace the argument list <args> via 'return <args>;'" );
+    argx =libGAP_NargError(self, libGAP_LEN_LIST(args));
     return libGAP_FuncCALL_FUNC_LIST( (libGAP_Obj)0, self, argx );
 }
 
@@ -1140,7 +1141,7 @@ libGAP_Obj libGAP_NewFunctionT (
     func = libGAP_NewBag( type, size );
 
     /* create a function with a fixed number of arguments                  */
-    if ( narg != -1 ) {
+    if ( narg >= 0 ) {
         libGAP_HDLR_FUNC(func,0) = libGAP_DoFail0args;
         libGAP_HDLR_FUNC(func,1) = libGAP_DoFail1args;
         libGAP_HDLR_FUNC(func,2) = libGAP_DoFail2args;
@@ -1154,20 +1155,21 @@ libGAP_Obj libGAP_NewFunctionT (
 
     /* create a function with a variable number of arguments               */
     else {
-        libGAP_HDLR_FUNC(func,0) = libGAP_DoWrap0args;
-        libGAP_HDLR_FUNC(func,1) = libGAP_DoWrap1args;
-        libGAP_HDLR_FUNC(func,2) = libGAP_DoWrap2args;
-        libGAP_HDLR_FUNC(func,3) = libGAP_DoWrap3args;
-        libGAP_HDLR_FUNC(func,4) = libGAP_DoWrap4args;
-        libGAP_HDLR_FUNC(func,5) = libGAP_DoWrap5args;
-        libGAP_HDLR_FUNC(func,6) = libGAP_DoWrap6args;
-        libGAP_HDLR_FUNC(func,7) = hdlr;
+      libGAP_HDLR_FUNC(func,0) = (narg >= -1) ? libGAP_DoWrap0args : libGAP_DoFail0args;
+      libGAP_HDLR_FUNC(func,1) = (narg >= -2) ? libGAP_DoWrap1args : libGAP_DoFail1args;
+      libGAP_HDLR_FUNC(func,2) = (narg >= -3) ? libGAP_DoWrap2args : libGAP_DoFail2args;
+      libGAP_HDLR_FUNC(func,3) = (narg >= -4) ? libGAP_DoWrap3args : libGAP_DoFail3args;
+      libGAP_HDLR_FUNC(func,4) = (narg >= -5) ? libGAP_DoWrap4args : libGAP_DoFail4args;
+      libGAP_HDLR_FUNC(func,5) = (narg >= -6) ? libGAP_DoWrap5args : libGAP_DoFail5args;
+      libGAP_HDLR_FUNC(func,6) = (narg >= -7) ? libGAP_DoWrap6args : libGAP_DoFail6args;
+      libGAP_HDLR_FUNC(func,7) = hdlr;
     }
 
-    /* enter the the arguments and the names                               */
+    /* enter the arguments and the names                               */
     libGAP_NAME_FUNC(func) = name;
     libGAP_NARG_FUNC(func) = narg;
     libGAP_NAMS_FUNC(func) = nams;
+    if (nams) libGAP_MakeBagPublic(nams);
     libGAP_CHANGED_BAG(func);
 
     /* enter the profiling bag                                             */
@@ -1265,9 +1267,9 @@ libGAP_Obj libGAP_ArgStringToList(const libGAP_Char *nams_c) {
 /****************************************************************************
 **
 
-*F  TypeFunction( <func> )  . . . . . . . . . . . . . . .  kind of a function
+*F  TypeFunction( <func> )  . . . . . . . . . . . . . . .  type of a function
 **
-**  'TypeFunction' returns the kind of the function <func>.
+**  'TypeFunction' returns the type of the function <func>.
 **
 **  'TypeFunction' is the function in 'TypeObjFuncs' for functions.
 */
@@ -1297,7 +1299,9 @@ void libGAP_PrintFunction (
     libGAP_Int                 nloc;           /* number of locals                */
     libGAP_Obj                 oldLVars;       /* terrible hack                   */
     libGAP_UInt                i;              /* loop variable                   */
+    libGAP_UInt                isvarg;         /* does function have varargs?     */
 
+    isvarg = 0;
 
     if ( libGAP_IS_OPERATION(func) ) {
       libGAP_CALL_1ARGS( libGAP_PrintOperation, func );
@@ -1308,12 +1312,20 @@ void libGAP_PrintFunction (
     libGAP_Pr("%5>function%< ( %>",0L,0L);
 
     /* print the arguments                                                 */
-    narg = (libGAP_NARG_FUNC(func) == -1 ? 1 : libGAP_NARG_FUNC(func));
+    narg = libGAP_NARG_FUNC(func);
+    if (narg < 0) {
+      isvarg = 1;
+      narg = -narg;
+    }
+    
     for ( i = 1; i <= narg; i++ ) {
         if ( libGAP_NAMS_FUNC(func) != 0 )
             libGAP_Pr( "%I", (libGAP_Int)libGAP_NAMI_FUNC( func, (libGAP_Int)i ), 0L );
         else
             libGAP_Pr( "<<arg-%d>>", (libGAP_Int)i, 0L );
+        if(isvarg && i == narg) {
+            libGAP_Pr("...", 0L, 0L);
+        }
         if ( i != narg )  libGAP_Pr("%<, %>",0L,0L);
     }
     libGAP_Pr(" %<)",0L,0L);
@@ -1339,7 +1351,7 @@ void libGAP_PrintFunction (
             libGAP_Pr("<<kernel or compiled code>>",0L,0L);
         }
         else {
-            libGAP_SWITCH_TO_NEW_LVARS( func, libGAP_NARG_FUNC(func), libGAP_NLOC_FUNC(func),
+            libGAP_SWITCH_TO_NEW_LVARS( func, narg, libGAP_NLOC_FUNC(func),
                                  oldLVars );
             libGAP_PrintStat( libGAP_FIRST_STAT_CURR_FUNC );
             libGAP_SWITCH_TO_OLD_LVARS( oldLVars );
@@ -1585,9 +1597,10 @@ libGAP_Obj libGAP_FuncSET_NAME_FUNC(
                       libGAP_Obj func,
                       libGAP_Obj name )
 {
-  while (!libGAP_IsStringConv(name))
+  while (!libGAP_IsStringConv(name)) {
     name = libGAP_ErrorReturnObj("SET_NAME_FUNC( <func>, <name> ): <name> must be a string, not a %s",
                           (libGAP_Int)libGAP_TNAM_OBJ(name), 0, "YOu can return a new name to continue");
+  }
   if (libGAP_TNUM_OBJ(func) == libGAP_T_FUNCTION ) {
     libGAP_NAME_FUNC(func) = name;
     libGAP_CHANGED_BAG(func);
@@ -1782,8 +1795,11 @@ libGAP_Obj libGAP_FuncFILENAME_FUNC(libGAP_Obj self, libGAP_Obj func) {
 
     if (libGAP_BODY_FUNC(func)) {
         libGAP_Obj fn =  libGAP_FILENAME_BODY(libGAP_BODY_FUNC(func));
-        if (fn)
+#ifndef WARD_ENABLED
+        if (fn) {
             return fn;
+        }
+#endif
     }
     return libGAP_Fail;
 }
@@ -2031,7 +2047,7 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_InfoBags[ libGAP_T_FUNCTION ].name = "function";
     libGAP_InitMarkFuncBags( libGAP_T_FUNCTION , libGAP_MarkAllSubBags );
 
-    /* install the kind function                                           */
+    /* install the type functions                                          */
     libGAP_ImportGVarFromLibrary( "TYPE_FUNCTION",  &libGAP_TYPE_FUNCTION  );
     libGAP_ImportGVarFromLibrary( "TYPE_OPERATION", &libGAP_TYPE_OPERATION );
     libGAP_TypeObjFuncs[ libGAP_T_FUNCTION ] = libGAP_TypeFunction;
@@ -2120,7 +2136,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoCalls ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
