@@ -51,6 +51,10 @@
 #include        "plist.h"               /* plain lists                     */
 #include        "string.h"              /* strings                         */
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+
 
 static int             libGAP_evlist, libGAP_evlistvec;
 
@@ -302,29 +306,6 @@ libGAP_Obj       libGAP_Multiplybound(
 
 
 
-/****************************************************************************
-**
-*F  FuncMultiply( <self>, <x>, <y>, <dtpols> )
-**
-**  FuncMultiply implements the internal function
-**
-*F  Multiply( <x>, <y>, <dtpols> ).
-**
-**  Multiply returns the product of the words <x> and <y> as ordered word
-**  by evaluating the deep thought polynomials <dtpols>.
-*/
-
-libGAP_Obj      libGAP_FuncMultiply(
-                       libGAP_Obj      self,
-                       libGAP_Obj      x,
-                       libGAP_Obj      y,
-                       libGAP_Obj      dtpols      )
-{
-    return libGAP_Multiplybound(x, y, 1, libGAP_LEN_PLIST(y), dtpols);
-}
-
-
-
 /****************************************************************************
 **
 *F  Power( <x>, <n>, <dtpols> )
@@ -390,29 +371,6 @@ libGAP_Obj      libGAP_Power(
 
 
 
-/****************************************************************************
-**
-*F  FuncPower( <self>, <x>, <n>, <dtpols> )
-**
-**  FuncPower implements the internal function
-**
-*F  Pover( <x>, <n>, <dtpols> )
-**
-**  Pover returns the <n>-th power of the word <x> by evaluating the deep
-**  thought pols <dtpols>. The result is an oredered word.
-*/
-
-libGAP_Obj        libGAP_FuncPower(
-                      libGAP_Obj     self,
-                      libGAP_Obj     x,
-                      libGAP_Obj     n,
-                      libGAP_Obj     dtpols     )
-{
-    return libGAP_Power(x, n, dtpols);
-}
-
-
-
 /****************************************************************************
 **
 *F  Solution( <x>, <y>, <dtpols> )
@@ -592,7 +550,7 @@ libGAP_Obj       libGAP_Conjugate( libGAP_Obj     x,
 **
 **  Multiplyboundred returns the product of the words <x> and <y>. The result
 **  is an ordered word with the additional property that all word exponents
-**  are reduced modulo the the corresponding generator orders given by the
+**  are reduced modulo the corresponding generator orders given by the
 **  deep thought rewriting system <pcp>..
 */
 
@@ -795,12 +753,12 @@ void     libGAP_compress( libGAP_Obj        list )
 
 /****************************************************************************
 **
-*F  Funccompress( <self>, <list> )
+*F  FuncDTCompress( <self>, <list> )
 **
-**  Funccompress implements the internal function Compress.
+**  FuncDTCompress implements the internal function DTCompress.
 */
 
-libGAP_Obj      libGAP_Funccompress( libGAP_Obj         self, 
+libGAP_Obj      libGAP_FuncDTCompress( libGAP_Obj         self, 
                        libGAP_Obj         list  )
 {
     libGAP_compress(list);
@@ -1067,14 +1025,8 @@ libGAP_Obj       libGAP_FuncDTQuotient( libGAP_Obj      self,
 */
 static libGAP_StructGVarFunc libGAP_GVarFuncs [] = {
 
-    { "Compress", 1, "list",
-      libGAP_Funccompress, "src/dteval.c:Compress" },
-
-    { "Multiply", 3, "lword, rword, representatives",
-      libGAP_FuncMultiply, "src/dteval.c:Multiply" },
-
-    { "Pover", 3, "word, exponent, representatives",
-      libGAP_FuncPower, "src/dteval.c:Pover" },
+    { "DTCompress", 1, "list",
+      libGAP_FuncDTCompress, "src/dteval.c:DTCompress" },
 
     { "DTMultiply", 3, "lword, rword, rws",
       libGAP_FuncDTMultiply, "src/dteval.c:DTMultiply" },
@@ -1166,7 +1118,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoDTEvaluation ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
