@@ -12,7 +12,6 @@
 **  This file implements the  functions  handling  GMP integers.
 **
 */
-#include        <sys/mman.h>
 #include        "system.h"              /* Ints, UInts                     */
 
 #include        "gasman.h"              /* garbage collector               */
@@ -57,6 +56,9 @@
 
 #ifdef USE_GMP
 
+/* TODO: Remove after Ward2 */
+#ifndef WARD_ENABLED
+
 // GMP must be included outside of 'extern C'
 #ifdef libGAP_GAP_IN_EXTERN_C
 }
@@ -68,6 +70,16 @@ extern "C" {
 
 #include        "gmpints.h"             /* GMP integers                    */
 
+#ifdef libGAP_SYS_IS_64_BIT
+#define libGAP_SaveLimb libGAP_SaveUInt8
+#define libGAP_LoadLimb libGAP_LoadUInt8
+#else
+#define libGAP_SaveLimb libGAP_SaveUInt4
+#define libGAP_LoadLimb libGAP_LoadUInt4
+#endif
+
+#define libGAP_INTBASE            (1L << (GMP_LIMB_BITS/2))
+
 
 /* macros to save typing later :)                                          */
 #define libGAP_VAL_LIMB0(obj)         ( *(libGAP_TypLimb *)libGAP_ADDR_OBJ(obj)                  )
@@ -140,7 +152,7 @@ libGAP_Obj libGAP_FuncIS_INT ( libGAP_Obj self, libGAP_Obj val )
        || libGAP_TNUM_OBJ(val) == libGAP_T_INTNEG ) {
     return libGAP_True;
   }
-  else if ( libGAP_TNUM_OBJ(val) <= libGAP_FIRST_EXTERNAL_TNUM ) {
+  else if ( libGAP_TNUM_OBJ(val) < libGAP_FIRST_EXTERNAL_TNUM ) {
     return libGAP_False;
   }
   else {
@@ -444,7 +456,7 @@ libGAP_Obj libGAP_FuncHexStringInt( libGAP_Obj self, libGAP_Obj integer )
     }
     
     /* else we create a string big enough for any immediate integer        */
-    res = libGAP_NEW_STRING(2 * libGAP_NR_HEX_DIGITS + 1);
+    res = libGAP_NEW_STRING(2 * libGAP_INTEGER_UNIT_SIZE + 1);
     p = libGAP_CHARS_STRING(res);
     /* handle sign */
     if (n<0) {
@@ -456,8 +468,8 @@ libGAP_Obj libGAP_FuncHexStringInt( libGAP_Obj self, libGAP_Obj integer )
       libGAP_SET_LEN_STRING(res, libGAP_GET_LEN_STRING(res)-1);
     /* collect digits, skipping leading zeros                              */
     j = 0;
-    nf = ((libGAP_UInt)15) << (4*(2*libGAP_NR_HEX_DIGITS-1));
-    for (i = 2*libGAP_NR_HEX_DIGITS; i; i-- ) {
+    nf = ((libGAP_UInt)15) << (4*(2*libGAP_INTEGER_UNIT_SIZE-1));
+    for (i = 2*libGAP_INTEGER_UNIT_SIZE; i; i-- ) {
       a = ((libGAP_UInt)n & nf) >> (4*(i-1));
       if (j==0 && a==0) libGAP_SET_LEN_STRING(res, libGAP_GET_LEN_STRING(res)-1);
       else if (a<10) p[j++] = a + '0';
@@ -565,8 +577,8 @@ libGAP_Obj libGAP_FuncIntHexString( libGAP_Obj self,  libGAP_Obj str )
   }
 
   else {
-    nd = (len-i)/libGAP_NR_HEX_DIGITS;
-    if (nd * libGAP_NR_HEX_DIGITS < (len-i)) nd++;
+    nd = (len-i)/libGAP_INTEGER_UNIT_SIZE;
+    if (nd * libGAP_INTEGER_UNIT_SIZE < (len-i)) nd++;
     /*   nd += ((3*nd) % 4); */
     if (sign == 1)
       res = libGAP_NewBag( libGAP_T_INTPOS, nd*sizeof(libGAP_TypLimb) );
@@ -599,6 +611,25 @@ libGAP_Obj libGAP_FuncIntHexString( libGAP_Obj self,  libGAP_Obj str )
   }
 }
 
+/****************************************************************************
+**  
+**  Implementation of Log2Int for C integers.
+*/
+
+libGAP_Int libGAP_CLog2Int(libGAP_Int a)
+{
+  libGAP_Int res, mask;
+  if (a < 0) a = -a;
+  if (a < 1) return -1;
+  if (a < 65536) {
+    for(mask = 2, res = 0; ;mask *= 2, res += 1) {
+      if(a < mask) return res;
+    }
+  }
+  for(mask = 65536, res = 15; ;mask *= 2, res += 1) {
+    if(a < mask) return res;
+  }
+}
 
 /****************************************************************************
 **
@@ -608,20 +639,13 @@ libGAP_Obj libGAP_FuncIntHexString( libGAP_Obj self,  libGAP_Obj str )
 */
 libGAP_Obj libGAP_FuncLog2Int( libGAP_Obj self, libGAP_Obj integer)
 {
-  libGAP_Int res, d;
+  libGAP_Int d;
   libGAP_Int a, len;
-  libGAP_Int mask;
   libGAP_TypLimb dmask;
   
   /* case of small ints                                                    */
   if (libGAP_IS_INTOBJ(integer)) {
-    a = libGAP_INT_INTOBJ(integer);
-    if (a < 0) a = -a;
-    res = libGAP_NR_SMALL_INT_BITS;
-    for(res = libGAP_NR_SMALL_INT_BITS - 1, mask = (libGAP_Int)1 << (libGAP_NR_SMALL_INT_BITS-1);
-        (mask & a) == 0 && mask != (libGAP_Int)0;
-        mask = mask >> 1, res--);
-    return libGAP_INTOBJ_INT(res);
+    return libGAP_INTOBJ_INT(libGAP_CLog2Int(libGAP_INT_INTOBJ(integer)));
   }
 
   /* case of long ints                                                     */
@@ -1270,6 +1294,7 @@ libGAP_Obj libGAP_ProdInt ( libGAP_Obj gmpL, libGAP_Obj gmpR )
   libGAP_Int                   i;            /* hold small int value              */
   libGAP_Int                   k;            /* hold small int value              */
   libGAP_TypLimb           carry;            /* most significant limb             */
+  libGAP_TypLimb      tmp1, tmp2;
   
   /* multiplying two small integers                                        */
   if ( libGAP_ARE_INTOBJS( gmpL, gmpR ) ) {
@@ -1295,8 +1320,9 @@ libGAP_Obj libGAP_ProdInt ( libGAP_Obj gmpL, libGAP_Obj gmpR )
     if ( k < 0 )  k = -k;
     
     /* multiply                                                            */
-    mpn_mul_n( libGAP_ADDR_INT( prd ), (libGAP_TypLimb*)( &i ),
-               (libGAP_TypLimb*)( &k ), (libGAP_TypGMPSize)1 );
+    tmp1 = (libGAP_TypLimb)i;
+    tmp2 = (libGAP_TypLimb)k;
+    mpn_mul_n( libGAP_ADDR_INT( prd ), &tmp1, &tmp2, 1 );
   }
   
   /* multiply a small and a large integer                                  */
@@ -2375,7 +2401,7 @@ libGAP_Obj libGAP_FuncRandomIntegerMT(libGAP_Obj self, libGAP_Obj mtstr, libGAP_
   libGAP_Obj res;
   libGAP_Int i, n, q, r, qoff, len;
   libGAP_UInt4 *mt, rand;
-  libGAP_TypLimb *pt;
+  libGAP_UInt4 *pt;
   while (! libGAP_IsStringConv(mtstr)) {
      mtstr = libGAP_ErrorReturnObj(
          "<mtstr> must be a string, not a %s)",
@@ -2415,30 +2441,24 @@ libGAP_Obj libGAP_FuncRandomIntegerMT(libGAP_Obj self, libGAP_Obj mtstr, libGAP_
 #endif
   }
   else {
-     /* large int case - number of Limbs */
-     q = n / GMP_LIMB_BITS;
-     r = n - q*GMP_LIMB_BITS;
+     /* large int case */
+     q = n / 32;
+     r = n - q * 32;
+     /* qoff = number of 32 bit words we need */
      qoff = q + (r==0 ? 0:1);
-     len = qoff;
+     /* len = number of limbs we need (limbs currently are either 32 or 64 bit wide) */
+     len = (qoff*4 +  sizeof(libGAP_TypLimb) - 1) / sizeof(libGAP_TypLimb);
      res = libGAP_NewBag( libGAP_T_INTPOS, len*sizeof(libGAP_TypLimb) );
-     pt = libGAP_ADDR_INT(res);
+     pt = (libGAP_UInt4*) libGAP_ADDR_INT(res);
      mt = (libGAP_UInt4*) libGAP_CHARS_STRING(mtstr);
-#ifdef libGAP_SYS_IS_64_BIT
      for (i = 0; i < qoff; i++, pt++) {
-       rand = (libGAP_TypLimb) libGAP_nextrandMT_int32(mt);
-       *pt = rand; 
-       rand = (libGAP_TypLimb) libGAP_nextrandMT_int32(mt);
-       *pt |= (libGAP_UInt)rand << 32; 
+       rand = (libGAP_UInt4) libGAP_nextrandMT_int32(mt);
+       *pt = rand;
      }
-#else
-     for (i = 0; i < qoff; i++, pt++) {
-       rand = libGAP_nextrandMT_int32(mt);
-       *pt = (libGAP_TypLimb) rand;
-     }
-#endif
      if (r != 0) {
-       libGAP_ADDR_INT(res)[qoff-1] = libGAP_ADDR_INT(res)[qoff-1] & ((libGAP_TypLimb)(-1)
-                                                      >> (GMP_LIMB_BITS-r));
+       /* we generated too many random bits -- chop of the extra bits */
+       pt = (libGAP_UInt4*) libGAP_ADDR_INT(res);
+       pt[qoff-1] = pt[qoff-1] & ((libGAP_UInt4)(-1) >> (32-r));
      }
      /* shrink bag if necessary */
      res = libGAP_GMP_NORMALIZE(res);
@@ -2523,23 +2543,6 @@ static libGAP_StructGVarFunc libGAP_GVarFuncs [] = {
 **
 *F  InitKernel( <module> )  . . . . . . . . initialise kernel data structures
 */
-
-static void *libGAP_allocForGmp(size_t size) {
-  return mmap((void *)0, size, PROT_READ| PROT_WRITE, MAP_ANON | MAP_PRIVATE, -1, 0);
-}
-
-static void *libGAP_reallocForGmp (void *old, size_t old_size, size_t new_size) {
-  void * newptr = libGAP_allocForGmp(new_size);
-  size_t common_size = (new_size < old_size) ? new_size:old_size;
-  memcpy(newptr, old, common_size);
-  munmap(old, old_size);
-  return newptr;
-}
-
-static  void libGAP_freeForGmp (void *ptr, size_t size) {
-  munmap(ptr, size);
-}
-
 static libGAP_Int libGAP_InitKernel ( libGAP_StructInitInfo * libGAP_module )
 {
   libGAP_UInt                t1,  t2;
@@ -2549,8 +2552,6 @@ static libGAP_Int libGAP_InitKernel ( libGAP_StructInitInfo * libGAP_module )
     libGAP_SyExit( 1 ); 
   }
 
-   mp_set_memory_functions( libGAP_allocForGmp, libGAP_reallocForGmp, libGAP_freeForGmp); 
-
   /* init filters and functions                                            */
   libGAP_InitHdlrFiltsFromTable( libGAP_GVarFilts );
   libGAP_InitHdlrFuncsFromTable( libGAP_GVarFuncs );
@@ -2636,11 +2637,14 @@ static libGAP_Int libGAP_InitKernel ( libGAP_StructInitInfo * libGAP_module )
   libGAP_ImportFuncFromLibrary( "String", &libGAP_String );
   libGAP_ImportFuncFromLibrary( "One", &libGAP_OneAttr);
 
-  /* install the kind functions                                          */
+  /* install the type functions                                          */
   libGAP_TypeObjFuncs[ libGAP_T_INT    ] = libGAP_TypeIntSmall;
   libGAP_TypeObjFuncs[ libGAP_T_INTPOS ] = libGAP_TypeIntLargePos;
   libGAP_TypeObjFuncs[ libGAP_T_INTNEG ] = libGAP_TypeIntLargeNeg;
 
+  libGAP_MakeBagTypePublic( libGAP_T_INTPOS );
+  libGAP_MakeBagTypePublic( libGAP_T_INTNEG );
+  
   /* return success                                                        */
   return 0;
 }
@@ -2691,12 +2695,11 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoInt ( void )
 {
-  libGAP_FillInVersion( &libGAP_module );
   return &libGAP_module;
 }
 
-/* corresponds to USE_GMP test at start */
-#endif
+#endif /* ! WARD_ENABLED */
+#endif /* USE_GMP */
 
 /****************************************************************************
 **
