@@ -36,6 +36,8 @@
 
 #include        "gap.h"                 /* error handling, initialisation  */
 
+#include        "code.h"                /* coder                           */
+
 #include        "gvars.h"               /* global variables                */
 
 #include        "calls.h"               /* generic call mechanism          */
@@ -50,9 +52,12 @@
 
 #include        "bool.h"                /* booleans                        */
 
+#include        "tls.h"                 /* thread-local storage            */
+#include        "thread.h"              /* threads                         */
+#include        "aobjects.h"            /* atomic objects                  */
+
 /****************************************************************************
 **
-
 *V  ValGVars  . . . . . . . . . . . . . . . . . .  values of global variables
 *V  PtrGVars  . . . . . . . . . . . . . pointer to values of global variables
 **
@@ -294,20 +299,19 @@ libGAP_Obj libGAP_NameGVarObj ( libGAP_UInt gvar )
     return libGAP_ELM_PLIST( libGAP_NameGVars, gvar );
 }
 
-
 #define libGAP_NSCHAR '@'
 
 libGAP_Obj libGAP_CurrNamespace = 0;
 
 libGAP_Obj libGAP_FuncSET_NAMESPACE(libGAP_Obj self, libGAP_Obj str)
 {
-    libGAP_CurrNamespace = str;
+    libGAP_TLS(libGAP_CurrNamespace) = str;
     return 0;
 }
 
 libGAP_Obj libGAP_FuncGET_NAMESPACE(libGAP_Obj self)
 {
-    return libGAP_CurrNamespace;
+    return libGAP_TLS(libGAP_CurrNamespace);
 }
 
 /****************************************************************************
@@ -332,7 +336,7 @@ libGAP_UInt libGAP_GVarName (
     libGAP_Int                 len;            /* length of name                  */
 
     /* First see whether it could be namespace-local: */
-    cns = libGAP_CSTR_STRING(libGAP_CurrNamespace);
+    cns = libGAP_CSTR_STRING(libGAP_TLS(libGAP_CurrNamespace));
     if (*cns) {   /* only if a namespace is set */
         len = strlen(name);
         if (name[len-1] == libGAP_NSCHAR) {
@@ -1191,8 +1195,8 @@ static libGAP_Int libGAP_InitLibrary (
     libGAP_SET_LEN_PLIST( libGAP_TableGVars, libGAP_SizeGVars );
 
     /* Create the current namespace: */
-    libGAP_CurrNamespace = libGAP_NEW_STRING(0);
-    libGAP_SET_LEN_STRING(libGAP_CurrNamespace,0);
+    libGAP_TLS(libGAP_CurrNamespace) = libGAP_NEW_STRING(0);
+    libGAP_SET_LEN_STRING(libGAP_TLS(libGAP_CurrNamespace),0);
     
     /* fix C vars                                                          */
     libGAP_PostRestore( libGAP_module );
@@ -1245,7 +1249,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoGVars ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
