@@ -45,6 +45,9 @@
 
 #include        "saveload.h"            /* saving and loading              */
 
+#include        "code.h"                /* coder                           */
+#include        "thread.h"              /* threads                         */
+#include        "tls.h"                 /* thread-local storage            */
 
 
 #include <stdio.h>
@@ -218,7 +221,7 @@ libGAP_Obj libGAP_FuncRandomListMT(libGAP_Obj self, libGAP_Obj mtstr, libGAP_Obj
   len = libGAP_LEN_LIST(list);
   if (len == 0) return libGAP_Fail;
   mt = (libGAP_UInt4*) libGAP_CHARS_STRING(mtstr);
-  lg = 31 - libGAP_INT_INTOBJ(libGAP_FuncLog2Int((libGAP_Obj)0, libGAP_INTOBJ_INT(len)));
+  lg = 31 - libGAP_CLog2Int(len);
   for (a = libGAP_nextrandMT_int32(mt) >> lg;
        a >= len;
        a = libGAP_nextrandMT_int32(mt) >> lg
@@ -230,11 +233,11 @@ libGAP_Obj libGAP_FuncRandomListMT(libGAP_Obj self, libGAP_Obj mtstr, libGAP_Obj
 
 /****************************************************************************
 **
-*F  FuncHASHKEY_BAG(<self>,<obj>,<factor>,<offset>,<maxlen>)
+*F  FuncHASHKEY_BAG(<self>,<obj>,<seed>,<offset>,<maxlen>)
 **
 **  'FuncHASHKEY_BAG' implements the internal function 'HASHKEY_BAG'.
 **
-**  'HASHKEY_BAG( <obj>, <factor>,<offset>,<maxlen> )'
+**  'HASHKEY_BAG( <obj>, <seed>, <offset>, <maxlen> )'
 **
 **  takes an non-immediate object and a small integer <int> and computes a
 **  hash value for the contents of the bag from these. (For this to be
@@ -581,71 +584,64 @@ void libGAP_MurmurHash3_x64_128 ( const void * key, const int len,
 }
 
 
-libGAP_Obj             libGAP_FuncHASHKEY_BAG (
-    libGAP_Obj                 self,
-    libGAP_Obj                 opL,
-    libGAP_Obj                 opR,
-    libGAP_Obj                 opO,
-    libGAP_Obj			opM)
+libGAP_Obj libGAP_FuncHASHKEY_BAG(libGAP_Obj self, libGAP_Obj obj, libGAP_Obj opSeed, libGAP_Obj opOffset, libGAP_Obj opMaxLen)
 {
   libGAP_Int n;
   libGAP_Int offs;
 
   /* check the arguments                                                 */
-  while ( libGAP_TNUM_OBJ(opR) != libGAP_T_INT ) {
-      opR = libGAP_ErrorReturnObj(
+  while ( libGAP_TNUM_OBJ(opSeed) != libGAP_T_INT ) {
+      opSeed = libGAP_ErrorReturnObj(
 	  "HASHKEY_BAG: <seed> must be a small integer (not a %s)",
-	  (libGAP_Int)libGAP_TNAM_OBJ(opR), 0L,
+	  (libGAP_Int)libGAP_TNAM_OBJ(opSeed), 0L,
 	  "you can replace <seed> via 'return <seed>;'" );
   }
   
   do {
     offs = -1;
 
-    while ( libGAP_TNUM_OBJ(opO) != libGAP_T_INT  ) {
-      opO = libGAP_ErrorReturnObj(
-			   "HASHKEY_BAG: <offset> must be a small integer (not a %s)",
-			   (libGAP_Int)libGAP_TNAM_OBJ(opO), 0L,
-			   "you can replace <offset> via 'return <offset>;'" );      
+    while ( libGAP_TNUM_OBJ(opOffset) != libGAP_T_INT  ) {
+      opOffset = libGAP_ErrorReturnObj(
+       "HASHKEY_BAG: <offset> must be a small integer (not a %s)",
+       (libGAP_Int)libGAP_TNAM_OBJ(opOffset), 0L,
+       "you can replace <offset> via 'return <offset>;'" );      
     }
-    offs = libGAP_INT_INTOBJ(opO);
-    if ( offs < 0 || offs > libGAP_SIZE_OBJ(opL)) {
-      opO = libGAP_ErrorReturnObj(
-			   "HashKeyBag: <offset> must be non-negative and less than the bag size",
-			   0L, 0L,
-			   "you can replace <offset> via 'return <offset>;'" );      
+    offs = libGAP_INT_INTOBJ(opOffset);
+    if ( offs < 0 || offs > libGAP_SIZE_OBJ(obj)) {
+      opOffset = libGAP_ErrorReturnObj(
+        "HashKeyBag: <offset> must be non-negative and less than the bag size",
+        0L, 0L,
+        "you can replace <offset> via 'return <offset>;'" );      
       offs = -1; 
     }
   } while (offs < 0);
-    
 
-
-  while ( libGAP_TNUM_OBJ(opM) != libGAP_T_INT ) {
-      opM = libGAP_ErrorReturnObj(
-	  "HASHKEY_BAG: <maxlen> must be a small integer (not a %s)",
-	  (libGAP_Int)libGAP_TNAM_OBJ(opM), 0L,
-	  "you can replace <maxlen> via 'return <maxlen>;'" );
+  while ( libGAP_TNUM_OBJ(opMaxLen) != libGAP_T_INT ) {
+      opMaxLen = libGAP_ErrorReturnObj(
+        "HASHKEY_BAG: <maxlen> must be a small integer (not a %s)",
+        (libGAP_Int)libGAP_TNAM_OBJ(opMaxLen), 0L,
+        "you can replace <maxlen> via 'return <maxlen>;'" );
   }
 
-  n=libGAP_SIZE_OBJ(opL)-offs;
+  n=libGAP_SIZE_OBJ(obj)-offs;
 
   /* maximal number of bytes to read */
-  libGAP_Int maxlen=libGAP_INT_INTOBJ(opM);
+  libGAP_Int maxlen=libGAP_INT_INTOBJ(opMaxLen);
   if ((n>maxlen)&&(maxlen!=-1)) {n=maxlen;}; 
   
-  return libGAP_INTOBJ_INT(libGAP_HASHKEY_BAG_NC( opL, (libGAP_UInt4)libGAP_INT_INTOBJ(opR), offs, (int) n));
+  return libGAP_INTOBJ_INT(libGAP_HASHKEY_BAG_NC( obj, (libGAP_UInt4)libGAP_INT_INTOBJ(opSeed), offs, (int) n));
 }
 
-libGAP_Int libGAP_HASHKEY_BAG_NC (libGAP_Obj obj, libGAP_UInt4 factor, libGAP_Int skip, int maxread){
+libGAP_Int libGAP_HASHKEY_BAG_NC (libGAP_Obj obj, libGAP_UInt4 seed, libGAP_Int skip, int maxread){
   #ifdef libGAP_SYS_IS_64_BIT
     libGAP_UInt8 hashout[2];
     libGAP_MurmurHash3_x64_128 ( (const void *)((libGAP_UChar *)libGAP_ADDR_OBJ(obj)+skip), 
-                           maxread, (libGAP_UInt4)factor, (void *) hashout);
+                           maxread, seed, (void *) hashout);
     return hashout[0] % (1UL << 60);
   #else
     libGAP_UInt4 hashout;
     libGAP_MurmurHash3_x86_32 ( (const void *)((libGAP_UChar *)libGAP_ADDR_OBJ(obj)+skip), 
-                          maxread, (libGAP_UInt4)factor, (void *) &hashout);
+                          maxread, seed, (void *) &hashout);
     return hashout % (1UL << 28);
   #endif
 }
@@ -741,7 +737,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoIntFuncs ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
