@@ -35,6 +35,7 @@
 #include        "bool.h"                /* booleans                        */
 
 #include        "permutat.h"            /* permutations                    */
+#include        "finfield.h"            /* finite fields                   */
 #include        "trans.h"               /* transformations                 */
 #include        "pperm.h"               /* partial perms                   */
 
@@ -43,6 +44,10 @@
 #include        "plist.h"               /* plain lists                     */
 #include        "set.h"                 /* plain sets                      */
 #include        "range.h"               /* ranges                          */
+#include	"code.h"
+#include	"thread.h"
+#include	"tls.h"
+#include	"aobjects.h"		/* atomic objects		   */
 
 #include                <string.h>
 #include                <stdlib.h> 
@@ -57,13 +62,26 @@
 **  The  list is  automatically extended to   make room for  the new element.
 **  'AddList' returns nothing, it is called only for its side effect.
 */
+void            libGAP_AddList3 (
+    libGAP_Obj                 list,
+    libGAP_Obj                 obj,
+    libGAP_Int                 pos)
+{
+    libGAP_Int                 len;
+    libGAP_Int                 i;
+    len = libGAP_LEN_LIST(list);
+    if (pos == (libGAP_Int) -1)
+      pos = len + 1;
+    for (i = len +1; i > pos; i--)
+      libGAP_ASS_LIST(list, i, libGAP_ELM_LIST(list, i-1));
+    libGAP_ASS_LIST( list, pos, obj );
+}
+
 void            libGAP_AddList (
     libGAP_Obj                 list,
-    libGAP_Obj                 obj )
+    libGAP_Obj                 obj)
 {
-    libGAP_Int                 pos;            /* position to assign to           */
-    pos = libGAP_LEN_LIST( list ) + 1;
-    libGAP_ASS_LIST( list, pos, obj );
+  libGAP_AddList3(list, obj, -1);
 }
 
 extern libGAP_Obj libGAP_FuncADD_LIST(
@@ -71,11 +89,19 @@ extern libGAP_Obj libGAP_FuncADD_LIST(
     libGAP_Obj                 list,
     libGAP_Obj                 obj );
 
-void            libGAP_AddPlist (
+extern libGAP_Obj libGAP_FuncADD_LIST3(
+    libGAP_Obj                 self,
     libGAP_Obj                 list,
-    libGAP_Obj                 obj )
+    libGAP_Obj                 obj,
+    libGAP_Obj			pos );
+
+
+void            libGAP_AddPlist3 (
+    libGAP_Obj                 list,
+    libGAP_Obj                 obj,
+    libGAP_Int 		pos )
 {
-    libGAP_Int                 pos;            /* position to assign to           */
+  libGAP_UInt len;
 
     if ( ! libGAP_IS_MUTABLE_PLIST(list) ) {
         list = libGAP_ErrorReturnObj(
@@ -86,47 +112,74 @@ void            libGAP_AddPlist (
         return;
     }
     /* in order to be optimistic when building list call assignment        */
-    pos = libGAP_LEN_PLIST( list ) + 1;
-    if ( pos == 1 ) {
+    len = libGAP_LEN_PLIST( list );
+    if (pos == (libGAP_Int)-1)
+      pos = len + 1;
+    if ( len == 0) {
         libGAP_AssPlistEmpty( list, pos, obj );
+	return;
     }
-    else {
-      libGAP_ASS_LIST( list, pos, obj);
-      /*  The code below is commented out and replaced by the line above, so
-          as, at the cost of one extra dispatch, to take advantage of the
-          special code in AssPlist<things> which maintain as much information
-          about denseness homogeneity, etc as possible */
-      /*        RetypeBag( list, T_PLIST );
-                GROW_PLIST( list, pos );
-                SET_LEN_PLIST( list, pos );
-                SET_ELM_PLIST( list, pos, obj );
-                CHANGED_BAG( list ); */
+    if (pos <= len) {
+      libGAP_GROW_PLIST(list, len+1);
+      libGAP_SET_LEN_PLIST(list, len+1);
+      memmove((void *)(libGAP_ADDR_OBJ(list) + pos+1),
+	      (const void *)(libGAP_ADDR_OBJ(list) + pos),
+	      (size_t)(sizeof(libGAP_Obj)*(len - pos + 1)));
     }
+    libGAP_ASS_LIST( list, pos, obj);
+}
+void            libGAP_AddPlist (
+    libGAP_Obj                 list,
+    libGAP_Obj                 obj)
+{
+
+  libGAP_AddPlist3(list, obj, -1);
 }
 
 libGAP_Obj libGAP_AddListOper;
 
-libGAP_Obj libGAP_FuncADD_LIST (
+libGAP_Obj libGAP_FuncADD_LIST3 (
     libGAP_Obj                 self,
     libGAP_Obj                 list,
-    libGAP_Obj                 obj )
+    libGAP_Obj                 obj,
+    libGAP_Obj                 pos)
 {
-    /* dispatch                                                            */
-    if ( libGAP_IS_PLIST( list ) ) {
-        libGAP_AddPlist( list, obj );
-    }
-    else if ( libGAP_TNUM_OBJ( list ) < libGAP_FIRST_EXTERNAL_TNUM ) {
-        libGAP_AddList( list, obj );
-    }
-    else {
-        libGAP_DoOperation2Args( self, list, obj );
-    }
+    /* dispatch                */
+  libGAP_Int ipos;
+  if (pos == (libGAP_Obj)0)
+    ipos = -1;
+  else if (libGAP_IS_INTOBJ(pos))
+    ipos = libGAP_INT_INTOBJ(pos);
+  else {
+    libGAP_DoOperation3Args( self, list,  obj, pos);
+    return (libGAP_Obj) 0;
+  }
+  if ( libGAP_IS_PLIST( list ) ) {
+    libGAP_AddPlist3( list, obj, ipos );
+  } else if ( libGAP_TNUM_OBJ( list ) < libGAP_FIRST_EXTERNAL_TNUM ) {
+    libGAP_AddList3( list, obj, ipos );
+  } else {
+    if (pos == 0)
+      libGAP_DoOperation2Args( self, list, obj );
+    else
+      libGAP_DoOperation3Args( self, list, obj, pos);
+  }
 
     /* return nothing                                                      */
     return (libGAP_Obj)0;
 }
 
 
+libGAP_Obj libGAP_FuncADD_LIST (
+		  libGAP_Obj self,
+		  libGAP_Obj list,
+		  libGAP_Obj obj)
+{
+  libGAP_FuncADD_LIST3(self, list, obj, (libGAP_Obj)0);
+  return (libGAP_Obj) 0;
+}
+		   
+
 /****************************************************************************
 **
 *F  RemList(<list>) . . . . . . . .  add an object to the end of a list
@@ -332,6 +385,7 @@ libGAP_Obj             libGAP_FuncAPPEND_LIST (
     return (libGAP_Obj)0;
 }
 
+
 /****************************************************************************
 **
 *F  POSITION_SORTED_LIST(<list>,<obj>)  . . . . find an object in a sorted list
@@ -492,7 +546,7 @@ libGAP_Obj             libGAP_FuncPOSITION_SORTED_COMP (
     /* check the first argument                                            */
     while ( ! libGAP_IS_SMALL_LIST(list) ) {
         list = libGAP_ErrorReturnObj(
-            "POSITION_SORTED_LISTComp: <list> must be a small list (not a %s)",
+            "POSITION_SORTED_COMP: <list> must be a small list (not a %s)",
             (libGAP_Int)libGAP_TNAM_OBJ(list), 0L,
             "you can replace <list> via 'return <list>;'" );
     }
@@ -500,7 +554,7 @@ libGAP_Obj             libGAP_FuncPOSITION_SORTED_COMP (
     /* check the third argument                                            */
     while ( libGAP_TNUM_OBJ( func ) != libGAP_T_FUNCTION ) {
         func = libGAP_ErrorReturnObj(
-            "POSITION_SORTED_LISTComp: <func> must be a function (not a %s)",
+            "POSITION_SORTED_COMP: <func> must be a function (not a %s)",
             (libGAP_Int)libGAP_TNAM_OBJ(func), 0L,
             "you can replace <func> via 'return <func>;'" );
     }
@@ -1924,8 +1978,11 @@ libGAP_Obj libGAP_FuncHORSPOOL_LISTS(libGAP_Obj self,libGAP_Obj wrep, libGAP_Obj
 */
 static libGAP_StructGVarOper libGAP_GVarOpers [] = {
 
-    { "ADD_LIST", 2, "list, val", &libGAP_AddListOper,
-      libGAP_FuncADD_LIST, "src/listfunc.c:ADD_LIST" },
+  /*    { "ADD_LIST", 2, "list, val", &AddListOper,
+	FuncADD_LIST, "src/listfunc.c:ADD_LIST" }, */
+
+    { "ADD_LIST", -1, "list, obj", &libGAP_AddListOper,
+      libGAP_DoOperation0Args, "src/listfunc.c:ADD_LIST" },
 
     { "REM_LIST", 1, "list", &libGAP_RemListOper,
       libGAP_FuncREM_LIST, "src/listfunc.c:REM_LIST" },
@@ -2018,9 +2075,16 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_StructInitInfo *    libGAP_module )
 {
     /* init filters and functions                                          */
+    /* ADD_LIST needs special consideration because we want distinct kernel
+       handlers for 2 and 3 arguments */
+    libGAP_InitHandlerFunc( libGAP_FuncADD_LIST, "src/listfunc.c:FuncADD_LIST" );
+    libGAP_InitHandlerFunc( libGAP_FuncADD_LIST3, "src/listfunc.c:FuncADD_LIST3" );
+
     libGAP_InitHdlrOpersFromTable( libGAP_GVarOpers );
     libGAP_InitHdlrFuncsFromTable( libGAP_GVarFuncs );
 
+
+    
     /* return success                                                      */
     return 0;
 }
@@ -2037,6 +2101,10 @@ static libGAP_Int libGAP_InitLibrary (
     libGAP_InitGVarOpersFromTable( libGAP_GVarOpers );
     libGAP_InitGVarFuncsFromTable( libGAP_GVarFuncs );
 
+    /* make and install the 'ADD_LIST' operation                           */
+    libGAP_HDLR_FUNC( libGAP_AddListOper, 2 ) = libGAP_FuncADD_LIST;
+    libGAP_HDLR_FUNC( libGAP_AddListOper, 3 ) = libGAP_FuncADD_LIST3;
+
     /* return success                                                      */
     return 0;
 }
@@ -2063,7 +2131,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoListFunc ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
