@@ -20,12 +20,13 @@
 **  components are always appended (record bags grow by a factor of 5/4
 **  if needed as plists do), already sorted rnams are stored by storing
 **  their negative value to indicate sortedness. The new entries will have
-**  positive rnams and can thus be distinguished. Every read access will 
+**  positive rnams and can thus be distinguished. Every read access will
 **  clean up the mess by sorting the new part and then merging the two
 **  sorted areas. After that, all rnams are negative indicating sortedness.
 **
 */
 #include        <stdlib.h>              /* for qsort */
+#include        <sys/time.h>            /* for gettimeofday() */
 #include        "system.h"              /* system dependent part           */
 
 
@@ -52,6 +53,11 @@
 
 #include        "saveload.h"            /* saving and loading              */
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+#include	"aobjects.h"		/* thread-local storage		   */
+
 
 /****************************************************************************
 **
@@ -63,9 +69,9 @@
 /****************************************************************************
 **
 
-*F  TypePRec( <rec> ) . . . . . . . . . . . . . . . .  kind of a plain record
+*F  TypePRec( <rec> ) . . . . . . . . . . . . . . . .  type of a plain record
 **
-**  'TypePRec' returns the kind of the plain record <rec>.
+**  'TypePRec' returns the type of the plain record <rec>.
 **
 **  'TypePRec' is the function in 'TypeObjFuncs' for plain records.
 */
@@ -136,7 +142,7 @@ libGAP_Int libGAP_IsCopyablePRecYes (
 */
 libGAP_Obj libGAP_NEW_PREC(libGAP_UInt len)
 {
-    libGAP_Obj o = libGAP_NewBag( libGAP_T_PREC, (len) * 2*sizeof(libGAP_Obj) + 2*sizeof(libGAP_Obj) ); 
+    libGAP_Obj o = libGAP_NewBag( libGAP_T_PREC, (len) * 2*sizeof(libGAP_Obj) + 2*sizeof(libGAP_Obj) );
     libGAP_SET_LEN_PREC(o,0);
     return o;
 }
@@ -151,15 +157,15 @@ libGAP_Int             libGAP_GrowPRec (
     libGAP_Obj                 rec,
     libGAP_UInt                need )
 {
-    libGAP_UInt                newsize, want, good;     
+    libGAP_UInt                newsize, want, good;
 
     /* check if big enough */
     want = (2*need+2)*sizeof(libGAP_Obj);
     if (libGAP_SIZE_OBJ(rec) >= want) return 0L;
 
-    
+
     /* find out how large the bag should become at least                   */
-    good = ((5 * libGAP_LEN_PREC(rec) + 3)/4 + 1) * 2 * sizeof(libGAP_Obj) + 2; 
+    good = ((5 * libGAP_LEN_PREC(rec) + 3)/4 + 1) * 2 * sizeof(libGAP_Obj) + 2;
 
     /* but maybe we need more                                              */
     newsize = (want < good) ? good : want;
@@ -321,7 +327,7 @@ void libGAP_MakeImmutablePRec( libGAP_Obj rec)
 
 
 /****************************************************************************
- * FindPRec( <rec>, <rnam>, <pos>, <cleanup> )  
+ * FindPRec( <rec>, <rnam>, <pos>, <cleanup> )
  *   . . . . . . . . . . . . . . . . . find a component name by binary search
  *
  * Searches rnam in rec, sets pos to the position where it is found (return
@@ -339,12 +345,12 @@ libGAP_UInt libGAP_FindPRec( libGAP_Obj rec, libGAP_UInt rnam, libGAP_UInt *pos,
     libGAP_UInt high;
 
     high = libGAP_LEN_PREC(rec);
-    if (high > 0 && (libGAP_Int) (libGAP_GET_RNAM_PREC(rec,high)) > 0) {  
+    if (high > 0 && (libGAP_Int) (libGAP_GET_RNAM_PREC(rec,high)) > 0) {
         /* DIRTY! Not everything sorted! */
         if (cleanup) {
             libGAP_SortPRecRNam(rec,0);
-            /* Note that this does not change the length and it cannot 
-             * trigger a garbage collection if cleanup is 1! 
+            /* Note that this does not change the length and it cannot
+             * trigger a garbage collection if cleanup is 1!
              * We do not want record accesses to trigger garbage
              * collections! */
         } else {
@@ -557,7 +563,7 @@ static int libGAP_PrecComparer(const void *a, const void *b)
     else return 1;
 }
 
-void libGAP_SortPRecRNam ( 
+void libGAP_SortPRecRNam (
     libGAP_Obj                 rec, int inplace )
 {
     libGAP_UInt len = libGAP_LEN_PREC(rec);
@@ -692,62 +698,6 @@ void libGAP_SortPRec ( libGAP_Obj rec )
 */
 
 
-/****************************************************************************
-**
-
-*F  MethodPRec( <rec>, <rnam> ) . . . . . . . . . . get a method for a record
-**
-**  'MethodPRec' returns  the function in the  component with the record name
-**  <rnam> in the  record in the component  with the name 'operations' (which
-**  really should be  called 'methods') of  the record <rec>  if this exists.
-**  Otherwise it return 0.
-*/
-libGAP_UInt libGAP_OperationsRNam;                    /* 'operations' record name        */
-libGAP_UInt libGAP_COMPONENTSRNam;                    /* COMPONENTS record name          */
-
-libGAP_Obj libGAP_MethodPRec (
-    libGAP_Obj                 rec,
-    libGAP_UInt                rnam )
-{
-    libGAP_Obj                 method;         /* method, result                  */
-    libGAP_Obj                 opers;          /* operations record               */
-    libGAP_UInt                i;              /* loop variable                   */
-
-    /* is <rec> a record?                                                  */
-    if ( ! libGAP_IS_PREC_REP(rec) )
-        return 0;
-
-    /* try to get the operations record                                    */
-    if (!libGAP_FindPRec(rec,libGAP_OperationsRNam,&i,1)) {
-        return 0;
-    }
-    opers = libGAP_GET_ELM_PREC( rec, i );
-
-    /* check for an Operations Record object */
-    if ( libGAP_TNUM_OBJ(opers) == libGAP_T_COMOBJ)
-      {
-        /* Make use of the fact the Component objects look like Precs */
-        if ( !libGAP_FindPRec(opers,libGAP_COMPONENTSRNam,&i,1) ) {
-          return 0;
-        }
-        opers = libGAP_GET_ELM_PREC( opers, i );
-      }
-
-
-    if ( ! libGAP_IS_PREC_REP(opers) ) {
-        return 0;
-    }
-
-    /* try to get the method                                               */
-    if ( !libGAP_FindPRec(opers,rnam,&i,1) ) {
-        return 0;
-    }
-    method = libGAP_GET_ELM_PREC( opers, i );
-
-    /* return the method                                                   */
-    return method;
-}
-
 void libGAP_PrintPathPRec (
     libGAP_Obj                 rec,
     libGAP_Int                 indx )
@@ -830,290 +780,12 @@ libGAP_Obj libGAP_FuncREC_NAMES_COMOBJ (
 
 /****************************************************************************
 **
-*F  FuncSUM_PREC( <self>, <left>, <right> ) . . . . . . .  sum of two records
-**
-**  'SumRec' returns the  sum  of the two   operands <left> and <right>.   At
-**  least one of the operands must be a plain record.
-**
-**  If  at least one of the  operands is a  record and has a '.operations.\+'
-**  method, than this  is called and its  result  is returned.
-*/
-libGAP_UInt libGAP_SumRNam;                           /* '+' record name                 */
-
-libGAP_Obj libGAP_FuncSUM_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_SumRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_SumRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncDIFF_PREC( <self>, <left>, <right> )  . . . difference of two records
-**
-**  'DiffRec' returns the difference of the two  operands <left> and <right>.
-**  At least one of the operands must be a plain record.
-**
-**  If  at least one  of the operands is  a record and has a '.operations.\-'
-**  method, then this  is called  and  its result is returned.
-*/
-libGAP_UInt libGAP_DiffRNam;                          /* '-' record name                 */
-
-libGAP_Obj libGAP_FuncDIFF_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;           /* operation                       */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_DiffRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_DiffRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncPROD_PREC( <self>, <left>, <right> )  . . . .  product of two records
-**
-**  'ProdRec' returns the product of the two operands <left> and <right>.  At
-**  least one of the operands must be a plain record.
-**
-**  If  at least one  of the operands is  a record and has a '.operations.\*'
-**  method, then this  is called  and  its result is returned.
-*/
-libGAP_UInt libGAP_ProdRNam;                          /* '*' record name                 */
-
-libGAP_Obj libGAP_FuncPROD_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_ProdRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_ProdRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncQUO_PREC( <self>, <left>, <right> ) . . . . . quotient of two records
-**
-**  'QuoRec' returns the quotient of the two operands <left> and <right>.  At
-**  least one of the operands must be a plain record.
-**
-**  If  at least one  of the operands is  a record and has a '.operations.\/'
-**  method, then this  is called  and  its result is returned.
-*/
-libGAP_UInt libGAP_QuoRNam;                           /* '/' record name                 */
-
-libGAP_Obj libGAP_FuncQUO_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_QuoRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_QuoRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncLQUO_PREC( <self>, <left>, <right> )  .  left quotient of two records
-**
-**  'LQuoPRec' returns the   left quotient  of  the two   operands <left> and
-**  <right>.  At least one of the operands must be a plain record.
-**
-**  If  at   least   one   of  the  operands     is a  record   and    has  a
-**  '.operations.LeftQuotient' method, then this is  called and its result is
-**  returned.
-*/
-libGAP_UInt libGAP_LQuoRNam;                          /* 'LeftQuotient' record name      */
-
-libGAP_Obj libGAP_FuncLQUO_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_LQuoRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_LQuoRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncPOW_PREC( <self>, <left>, <right> ) . . . . . .  power of two records
-**
-**  'PowPRec' returns the power of  the two operands  <left> and <right>.  At
-**  least one of the operands must be a plain record.
-**
-**  If  at least one  of the operands is  a record and has a '.operations.\^'
-**  method, then this  is called  and  its result is returned.
-*/
-libGAP_UInt libGAP_PowRNam;                           /* '^' record name                 */
-
-libGAP_Obj libGAP_FuncPOW_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_PowRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_PowRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncCOMM_PREC( <self>, <left>, <right> )  . .   commutator of two records
-**
-**  'CommPRec' returns the commutator of the two operands <left> and <right>.
-**  At least one of the operands must be a plain record.
-**
-**  If at least one of the operands is  a record and has a '.operations.Comm'
-**  method, then this  is called and its  result  is returned.
-*/
-libGAP_UInt libGAP_CommRNam;                          /* 'Comm' record name              */
-
-libGAP_Obj libGAP_FuncCOMM_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_CommRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_CommRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncMOD_PREC( <self>, <left>, <right> ) . . . .  remainder of two records
-**
-**  'ModPRec' returns the   remainder  the operands  <left>   by  the operand
-**  <right>.  At least one of the operands must be a plain record.
-**
-**  If at least one of the operands is a  record and has a '.operations.\mod'
-**  method, then this  is  called and its  result is  returned.
-*/
-libGAP_UInt libGAP_ModRNam;                /* 'mod' record name               */
-
-libGAP_Obj libGAP_FuncMOD_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_ModRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_ModRNam )) )
-    {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncIN_PREC( <self>, <left>, <right> )  . . . membership test for records
-**
-**  'InRec' returns 'True' if the operand <left>  is an element of the record
-**  <right>.  <right> must be a plain record.
-**
-**  If <right> has  a '.operations.\in' method, than this  is called  and its
-**  result is returned.
-*/
-libGAP_UInt libGAP_InRNam;                            /* 'in' record name                */
-
-libGAP_Obj libGAP_FuncIN_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_InRNam )) ) {
-        return libGAP_TRY_NEXT_METHOD;
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncEQ_PREC_DEFAULT( <self>, <left>, <right> )  comparison of two records
+*F  FuncEQ_PREC( <self>, <left>, <right> )  . . . . comparison of two records
 **
 **  'EqRec' returns '1L'  if the two  operands <left> and <right> are equal
 **  and '0L' otherwise.  At least one operand must be a plain record.
 */
-libGAP_Obj libGAP_FuncEQ_PREC_DEFAULT (
+libGAP_Obj libGAP_FuncEQ_PREC (
     libGAP_Obj                 self,
     libGAP_Obj                 left,
     libGAP_Obj                 right )
@@ -1128,7 +800,9 @@ libGAP_Obj libGAP_FuncEQ_PREC_DEFAULT (
     if ( libGAP_LEN_PREC(left) != libGAP_LEN_PREC(right) )
         return libGAP_False;
 
-    /* Records are now always sorted by their RNam */
+    /* ensure records are sorted by their RNam */
+    libGAP_SortPRecRNam(left,0);
+    libGAP_SortPRecRNam(right,0);
 
     /* compare componentwise                                               */
     for ( i = 1; i <= libGAP_LEN_PREC(right); i++ ) {
@@ -1151,45 +825,13 @@ libGAP_Obj libGAP_FuncEQ_PREC_DEFAULT (
 
 /****************************************************************************
 **
-*F  FuncEQ_PREC( <self>, <left>, <right> )  . . . . comparison of two records
-**
-**  'EqRec' returns '1L'  if the two  operands <left> and <right> are equal
-**  and '0L' otherwise.  At least one operand must be a plain record.
-**
-**  If at  least one of the  operands is a  record and has a '.operations.\='
-**  method,  than this is called  and its result  is returned.  Otherwise the
-**  records are compared componentwise.
-*/
-libGAP_UInt libGAP_EqRNam;                            /* '=' record name                 */
-
-libGAP_Obj libGAP_FuncEQ_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_EqRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_EqRNam )) )
-    {
-        return libGAP_FuncEQ_PREC_DEFAULT( self, left, right );
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
-/****************************************************************************
-**
-*F  FuncLT_PREC_DEFAULT( <self>, <left>, <right> )  comparison of two records
+*F  FuncLT_PREC( <self>, <left>, <right> )   . . .  comparison of two records
 **
 **  'LtRec' returns '1L'  if the operand  <left> is  less than the  operand
 **  <right>, and '0L'  otherwise.  At least  one operand  must be a  plain
 **  record.
 */
-libGAP_Obj libGAP_FuncLT_PREC_DEFAULT (
+libGAP_Obj libGAP_FuncLT_PREC (
     libGAP_Obj                 self,
     libGAP_Obj                 left,
     libGAP_Obj                 right )
@@ -1202,7 +844,9 @@ libGAP_Obj libGAP_FuncLT_PREC_DEFAULT (
         if ( libGAP_TNUM_OBJ(left ) > libGAP_TNUM_OBJ(right) )  return libGAP_False;
     }
 
-    /* Records are now always sorted by their RNam */
+    /* ensure records are sorted by their RNam */
+    libGAP_SortPRecRNam(left,0);
+    libGAP_SortPRecRNam(right,0);
 
     /* compare componentwise                                               */
     for ( i = 1; i <= libGAP_LEN_PREC(right); i++ ) {
@@ -1236,39 +880,6 @@ libGAP_Obj libGAP_FuncLT_PREC_DEFAULT (
 }
 
 
-/****************************************************************************
-**
-*F  FuncLT_PREC( <self>, <left>, <right> )   . . .  comparison of two records
-**
-**  'LtRec' returns '1L'  if the operand  <left> is  less than the  operand
-**  <right>, and '0L'  otherwise.  At least  one operand  must be a  plain
-**  record.
-**
-**  If at least  one of the operands is  a record and  has a '.operations.\<'
-**  method, than this  is called and its  result is  returned.  Otherwise the
-**  records are compared componentwise.
-*/
-libGAP_UInt libGAP_LtRNam;                            /* '<' record name                 */
-
-libGAP_Obj libGAP_FuncLT_PREC (
-    libGAP_Obj                 self,
-    libGAP_Obj                 left,
-    libGAP_Obj                 right )
-{
-    libGAP_Obj                 method;         /* method                          */
-
-    /* try to find an applicable method                                    */
-    if ( ! (method = libGAP_MethodPRec( right, libGAP_LtRNam ))
-      && ! (method = libGAP_MethodPRec( left,  libGAP_LtRNam )) )
-    {
-        return libGAP_FuncLT_PREC_DEFAULT( self, left, right );
-    }
-
-    /* call that function                                                  */
-    return libGAP_CALL_2ARGS( method, left, right );
-}
-
-
 /****************************************************************************
 **
 *F  SavePRec( <prec> )
@@ -1339,45 +950,12 @@ static libGAP_StructGVarFunc libGAP_GVarFuncs [] = {
     { "REC_NAMES_COMOBJ", 1, "rec obj",
       libGAP_FuncREC_NAMES_COMOBJ, "src/precord.c:REC_NAMES_COMOBJ" },
 
-    { "SUM_PREC", 2, "left, right",
-      libGAP_FuncSUM_PREC, "src/precord.c:SUM_PREC" },
-
-    { "DIFF_PREC", 2, "left, right",
-      libGAP_FuncDIFF_PREC, "src/precord.c:DIFF_PREC" },
-
-    { "PROD_PREC", 2, "left, right",
-      libGAP_FuncPROD_PREC, "src/precord.c:PROD_PREC" },
-
-    { "QUO_PREC", 2, "left, right",
-      libGAP_FuncQUO_PREC, "src/precord.c:QUO_PREC" },
-
-    { "LQUO_PREC", 2, "left, right",
-      libGAP_FuncLQUO_PREC, "src/precord.c:LQUO_PREC" },
-
-    { "POW_PREC", 2, "left, right",
-      libGAP_FuncPOW_PREC, "src/precord.c:POW_PREC" },
-
-    { "MOD_PREC", 2, "left, right",
-      libGAP_FuncMOD_PREC, "src/precord.c:MOD_PREC" },
-
-    { "COMM_PREC", 2, "left, right",
-      libGAP_FuncCOMM_PREC, "src/precord.c:COMM_PREC" },
-
-    { "IN_PREC", 2, "left, right",
-      libGAP_FuncIN_PREC, "src/precord.c:IN_PREC" },
-
     { "EQ_PREC", 2, "left, right",
       libGAP_FuncEQ_PREC, "src/precord.c:EQ_PREC" },
 
-    { "EQ_PREC_DEFAULT",2,"left, right",
-      libGAP_FuncEQ_PREC_DEFAULT, "src/precord.c:EQ_PREC_DEFAULT" },
-
     { "LT_PREC", 2, "left, right",
       libGAP_FuncLT_PREC, "src/precord.c:LT_PREC" },
 
-    { "LT_PREC_DEFAULT",2,"left, right",
-      libGAP_FuncLT_PREC_DEFAULT, "src/precord.c:LT_PREC_DEFAULT" },
-
     { 0 }
 
 };
@@ -1402,6 +980,9 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_InitMarkFuncBags( libGAP_T_PREC            +libGAP_COPYING , libGAP_MarkAllSubBags );
     libGAP_InitMarkFuncBags( libGAP_T_PREC +libGAP_IMMUTABLE +libGAP_COPYING , libGAP_MarkAllSubBags );
 
+    /* Immutable records are public					   */
+    libGAP_MakeBagTypePublic( libGAP_T_PREC + libGAP_IMMUTABLE );
+
     /* init filters and functions                                          */
     libGAP_InitHdlrFuncsFromTable( libGAP_GVarFuncs );
 
@@ -1443,7 +1024,7 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_PrintPathFuncs[ libGAP_T_PREC            ] = libGAP_PrintPathPRec;
     libGAP_PrintPathFuncs[ libGAP_T_PREC +libGAP_IMMUTABLE ] = libGAP_PrintPathPRec;
 
-    /* install the kind function                                           */
+    /* install the type functions                                          */
     libGAP_ImportGVarFromLibrary( "TYPE_PREC_MUTABLE",   &libGAP_TYPE_PREC_MUTABLE   );
     libGAP_ImportGVarFromLibrary( "TYPE_PREC_IMMUTABLE", &libGAP_TYPE_PREC_IMMUTABLE );
 
@@ -1457,33 +1038,6 @@ static libGAP_Int libGAP_InitKernel (
 }
 
 
-/****************************************************************************
-<**
-*F  PostRestore( <module> ) . . . . . . . . . . . . . after restore workspace
-*/
-static libGAP_Int libGAP_PostRestore (
-    libGAP_StructInitInfo *    libGAP_module )
-{
-    /* get the appropriate record record name                              */
-    libGAP_OperationsRNam = libGAP_RNamName( "operations"   );
-    libGAP_COMPONENTSRNam = libGAP_RNamName( "COMPONENTS"   );
-    libGAP_EqRNam         = libGAP_RNamName( "="            );
-    libGAP_LtRNam         = libGAP_RNamName( "<"            );
-    libGAP_InRNam         = libGAP_RNamName( "in"           );
-    libGAP_SumRNam        = libGAP_RNamName( "+"            );
-    libGAP_DiffRNam       = libGAP_RNamName( "-"            );
-    libGAP_ProdRNam       = libGAP_RNamName( "*"            );
-    libGAP_QuoRNam        = libGAP_RNamName( "/"            );
-    libGAP_LQuoRNam       = libGAP_RNamName( "LeftQuotient" );
-    libGAP_PowRNam        = libGAP_RNamName( "^"            );
-    libGAP_CommRNam       = libGAP_RNamName( "Comm"         );
-    libGAP_ModRNam        = libGAP_RNamName( "mod"          );
-
-    /* return success                                                      */
-    return 0;
-}
-
-
 /****************************************************************************
 **
 *F  InitLibrary( <module> ) . . . . . . .  initialise library data structures
@@ -1495,7 +1049,7 @@ static libGAP_Int libGAP_InitLibrary (
     libGAP_InitGVarFuncsFromTable( libGAP_GVarFuncs );
 
     /* return success                                                      */
-    return libGAP_PostRestore( libGAP_module );
+    return 0;
 }
 
 
@@ -1515,12 +1069,11 @@ static libGAP_StructInitInfo libGAP_module = {
     0,                                  /* checkInit                      */
     0,                                  /* preSave                        */
     0,                                  /* postSave                       */
-    libGAP_PostRestore                         /* postRestore                    */
+    0                                   /* postRestore                    */
 };
 
 libGAP_StructInitInfo * libGAP_InitInfoPRecord ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
