@@ -70,17 +70,11 @@
 #include        "string.h"              /* strings                         */
 
 #include        "saveload.h"            /* saving and loading              */
+#include	"code.h"
+#include	"thread.h"
+#include	"tls.h"
 
 
-/****************************************************************************
-**
-
-*F  NUM_RAT(<rat>)  . . . . . . . . . . . . . . . . . numerator of a rational
-*F  DEN_RAT(<rat>)  . . . . . . . . . . . . . . . . denominator of a rational
-*/
-#define libGAP_NUM_RAT(rat)    libGAP_ADDR_OBJ(rat)[0]
-#define libGAP_DEN_RAT(rat)    libGAP_ADDR_OBJ(rat)[1]
-
 #if 0
 #define libGAP_CHECK_RAT(rat) if (libGAP_TNUM_OBJ(rat) == libGAP_T_RAT && \
                            libGAP_GcdInt(libGAP_NUM_RAT(rat),libGAP_DEN_RAT(rat)) != libGAP_INTOBJ_INT(1)) \
@@ -92,9 +86,9 @@
 
 /****************************************************************************
 **
-*F  TypeRat( <rat> )  . . . . . . . . . . . . . . . . . .  kind of a rational
+*F  TypeRat( <rat> )  . . . . . . . . . . . . . . . . . .  type of a rational
 **
-**  'TypeRat' returns the kind of the rational <rat>.
+**  'TypeRat' returns the type of the rational <rat>.
 **
 **  'TypeRat' is the function in 'TypeObjFuncs' for rationals.
 */
@@ -678,9 +672,6 @@ libGAP_Obj             libGAP_PowRat (
     libGAP_Obj                 pow;            /* power                           */
 
     libGAP_CHECK_RAT(opL);
-    /* raise numerator and denominator separately                          */
-    numP = libGAP_PowInt( libGAP_NUM_RAT(opL), opR );
-    denP = libGAP_PowInt( libGAP_DEN_RAT(opL), opR );
 
     /* if <opR> == 0 return 1                                              */
     if ( opR == libGAP_INTOBJ_INT( 0L ) ) {
@@ -710,8 +701,9 @@ libGAP_Obj             libGAP_PowRat (
 
     /* if <opR> is negative and numerator is -1 return (-1)^r * num(l)     */
     else if ( libGAP_NUM_RAT(opL) == libGAP_INTOBJ_INT( -1L ) ) {
-        pow = libGAP_ProdInt(libGAP_PowInt(libGAP_NUM_RAT(opL),libGAP_ProdInt(libGAP_INTOBJ_INT(-1L),opR)),
-                      libGAP_PowInt(libGAP_DEN_RAT(opL),libGAP_ProdInt(libGAP_INTOBJ_INT(-1L),opR)));
+        numP = libGAP_PowInt( libGAP_NUM_RAT(opL), libGAP_ProdInt( libGAP_INTOBJ_INT( -1L ), opR ) );
+        denP = libGAP_PowInt( libGAP_DEN_RAT(opL), libGAP_ProdInt( libGAP_INTOBJ_INT( -1L ), opR ) );
+        pow = libGAP_ProdInt(numP, denP);
     }
 
     /* if <opR> is negative do both powers, take care of the sign          */
@@ -905,9 +897,17 @@ static libGAP_Int libGAP_InitKernel (
 {
     /* install the marking function                                        */
     libGAP_InfoBags[         libGAP_T_RAT ].name = "rational";
+    /* MarkTwoSubBags() is faster for Gasman, but MarkAllSubBags() is
+     * more space-efficient for the Boehm GC and does not incur a
+     * speed penalty.
+     */
+#ifndef BOEHM_GC
     libGAP_InitMarkFuncBags( libGAP_T_RAT, libGAP_MarkTwoSubBags );
+#else
+    libGAP_InitMarkFuncBags( libGAP_T_RAT, libGAP_MarkAllSubBags );
+#endif
 
-    /* install the kind function                                           */
+    /* install the type functions                                          */
     libGAP_ImportGVarFromLibrary( "TYPE_RAT_POS", &libGAP_TYPE_RAT_POS );
     libGAP_ImportGVarFromLibrary( "TYPE_RAT_NEG", &libGAP_TYPE_RAT_NEG );
 
@@ -917,7 +917,7 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_InitHdlrFiltsFromTable( libGAP_GVarFilts );
     libGAP_InitHdlrFuncsFromTable( libGAP_GVarFuncs );
 
-    /* install a saving function */
+    /* install a saving functions */
     libGAP_SaveObjFuncs[ libGAP_T_RAT ] = libGAP_SaveRat;
     libGAP_LoadObjFuncs[ libGAP_T_RAT ] = libGAP_LoadRat;
 
@@ -1000,6 +1000,8 @@ static libGAP_Int libGAP_InitKernel (
     libGAP_PowFuncs [ libGAP_T_RAT    ][ libGAP_T_INTPOS ] = libGAP_PowRat;
     libGAP_PowFuncs [ libGAP_T_RAT    ][ libGAP_T_INTNEG ] = libGAP_PowRat;
 
+    libGAP_MakeBagTypePublic(libGAP_T_RAT);
+
     /* return success                                                      */
     return 0;
 }
@@ -1042,7 +1044,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoRat ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
