@@ -36,6 +36,10 @@
 #include        "plist.h"               /* plain lists                     */
 #include        "string.h"              /* strings                         */
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+
 
 /****************************************************************************
 **
@@ -79,6 +83,7 @@ libGAP_UInt            libGAP_RNamName (
 {
     libGAP_Obj                 rnam;           /* record name (as imm intobj)     */
     libGAP_UInt                pos;            /* hash position                   */
+    libGAP_UInt                len;            /* length of name                  */
     libGAP_Char                namx [1024];    /* temporary copy of <name>        */
     libGAP_Obj                 string;         /* temporary string object <name>  */
     libGAP_Obj                 table;          /* temporary copy of <HashRNam>    */
@@ -88,11 +93,17 @@ libGAP_UInt            libGAP_RNamName (
 
     /* start looking in the table at the following hash position           */
     pos = 0;
+    len = 0;
     for ( p = name; *p != '\0'; p++ ) {
         pos = 65599 * pos + *p;
+        len++;
     }
     pos = (pos % libGAP_SizeRNam) + 1;
 
+    if(len >= 1023) {
+        // Note: We can't pass 'name' here, as it might get moved by garbage collection
+        libGAP_ErrorQuit("Record names must consist of less than 1023 characters", 0, 0);
+    }
     /* look through the table until we find a free slot or the global      */
     while ( (rnam = libGAP_ELM_PLIST( libGAP_HashRNam, pos )) != 0
          && strncmp( libGAP_NAME_RNAM( libGAP_INT_INTOBJ(rnam) ), name, 1023 ) ) {
@@ -724,11 +735,13 @@ static libGAP_Int libGAP_InitLibrary (
     /* make the list of names of record names                              */
     libGAP_CountRNam = 0;
     libGAP_NamesRNam = libGAP_NEW_PLIST( libGAP_T_PLIST, 0 );
+    libGAP_MakeBagPublic(libGAP_NamesRNam);
     libGAP_SET_LEN_PLIST( libGAP_NamesRNam, 0 );
 
     /* make the hash list of record names                                  */
     libGAP_SizeRNam = 997;
     libGAP_HashRNam = libGAP_NEW_PLIST( libGAP_T_PLIST, libGAP_SizeRNam );
+    libGAP_MakeBagPublic(libGAP_HashRNam);
     libGAP_SET_LEN_PLIST( libGAP_HashRNam, libGAP_SizeRNam );
 
     /* init filters and functions                                          */
@@ -762,7 +775,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoRecords ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
