@@ -33,6 +33,10 @@
 
 #include        "saveload.h"            /* saving and loading              */
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+
 
 /***************************************************************************
 **
@@ -526,7 +530,7 @@ static FILE *libGAP_file;
 
 static void libGAP_report( libGAP_Bag bag)
 {
-  fprintf(libGAP_file,"%li %li\n", (libGAP_Int) libGAP_TNUM_BAG(bag), (libGAP_Int) libGAP_SIZE_BAG(bag));
+  fprintf(libGAP_file,"%li %li\n", (long) libGAP_TNUM_BAG(bag), (long) libGAP_SIZE_BAG(bag));
 }
 
 libGAP_Obj libGAP_BagStats(libGAP_Obj self, libGAP_Obj filename)
@@ -1073,7 +1077,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoSaveLoad ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
