@@ -44,6 +44,10 @@
 
 #include        "range.h"               /* ranges                          */
 
+#include	"code.h"		/* coder                           */
+#include	"thread.h"		/* threads			   */
+#include	"tls.h"			/* thread-local storage		   */
+
 #include <assert.h>
 
 #define libGAP_IS_IMM_PLIST(list)  ((libGAP_TNUM_OBJ(list) - libGAP_T_PLIST) % 2)
@@ -792,7 +796,6 @@ static libGAP_StructInitInfo libGAP_module = {
 
 libGAP_StructInitInfo * libGAP_InitInfoVector ( void )
 {
-    libGAP_FillInVersion( &libGAP_module );
     return &libGAP_module;
 }
 
